-- boilbot
-- boil bot a mining bot

-- Copyright (c) 2008 Stefan Scheffler

-- Permission is hereby granted, free of charge, to any person
-- obtaining a copy of this software and associated documentation
-- files (the "Software"), to deal in the Software without
-- restriction, including without limitation the rights to use,
-- copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following
-- conditions:

-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-- OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.

local _

local dullbot = dofile("dullbot/dullbot.lua")

local DEFAULT_TIME = 60
local DEFAULT_FIRECOMMAND = "+Shoot2"

local bot = dullbot.new()

local boilbot = {}

function boilbot.new()
	local instance = {}
	
	local bot = dullbot.new()

	local mined = {}

	local running = false
	
	local flymode = 0

	local brake_timer = Timer()
	local mine_timer = Timer()

	-- options
	local opt_time = DEFAULT_TIME
	local opt_firecommand = DEFAULT_FIRECOMMAND

	instance.dullbot = bot

	local function mine()
		-- XXX is this blocking?
		JettisonAll()

		gkinterface.GKProcessCommand(opt_firecommand)

		local minetime = os.time()
		local mining = true

		local function mine_timer_cb()
			JettisonAll()

			if (os.time() - minetime) > opt_time and mining then
				gkinterface.GKProcessCommand(opt_firecommand)
				mining = false
			elseif not GetTargetDistance() or 
			   GetTargetDistance() > 30 or 
		           (os.time() - minetime) > (opt_time + 2)
		        then
				print("done with this roid")
				gkinterface.GKProcessCommand(opt_firecommand.." 0")

				-- lost target or got too far away
				instance:find_roid()
				return
			else
				gkinterface.GKProcessCommand(opt_firecommand.." 0")
			end

			mine_timer:SetTimeout(500)
		end
		mine_timer:SetTimeout(500, mine_timer_cb)
	end

	local function target_next_roid()
		local target
		local target_distance = 2^32

		local node, prev = radar.GetRadarSelectionID()
		if node ~= 2 then
			prev = nil
		end
		
		for i=1,1000 do
			radar.SetRadarSelection(2, i)

			local _, current = radar.GetRadarSelectionID()

			if prev ~= current then
				if not mined[i] and 
				   (GetTargetInfo() == "Asteroid" or GetTargetInfo() == "Ice Crystal") and
				   GetTargetDistance() < target_distance 
			        then
					target = i
					target_distance = GetTargetDistance()
				end
			end
			
			prev = current
		end
		
		if target then
			mined[target] = true
			radar.SetRadarSelection(2, target)
			return true
		end
	end

	function instance:cleanup()
		brake_timer:Kill()
		mine_timer:Kill()

		if bot:IsRunning() then
			bot:Abort()
		end

		Game.SetCVar("flymodeflag", flymode)
		Game.EnableInput()

		running = false
	end

	-- find next roid and mine it
	function instance:find_roid()
		-- target next unmined roid
		-- if none left finish up
		if not target_next_roid() then
			print("no roid in range. stopping")
			self:cleanup()
			return
		end

		local function bot_cb(err)
			Game.SetDesiredSpeed(0)

			if err then
				print("dullbot error: "..err)
				self:cleanup()
			else
				brake_timer:SetTimeout(5000, mine)
			end
		end

		local err = bot:Init(bot_cb)

		if err then
			print("dullbot error "..err)
			self:cleanup()
		end
	end


	-- interface
	-- options
	function instance:ResetConfig()
		opt_time = DEFAULT_TIME
	end

	function instance:SetMineTime(time)
		opt_time = time
	end

	function instance:FireButton(button)
		if button == 1 then
			opt_firecommand = "+Shoot2"
		elseif button == 2 then
			opt_firecommand = "+Shoot1"
		elseif button == 3 then
			opt_firecommand = "+Shoot3"
		else
			print("invalid fire button")
		end
	end
	
	-- start the bot
	function instance:Start()
		running = true
		
		mined = {}

		-- save original flymode
		flymode = Game.GetCVar("flymodeflag")

		-- put game into the correct state
		Game.SetCVar("flymodeflag", 1)
		gkinterface.GKProcessCommand(opt_firecommand.." 0")
		Game.DisableInput()

		-- setup dullbot
		bot:ResetConfig()

		bot:NoTurboZone(700)
		bot:Tolerance(0.2)

		-- this layer takes care of input
		bot:SetDisableInput(false)

		bot:SetApproachMode(50)
		bot:MoveDuration(10)

		self:find_roid()
	end

	-- stop it
	function instance:Abort()
		print("aborting...")
		self:cleanup()
	end

	function instance:IsRunning()
		return running
	end

	return instance
end

return boilbot
