-- addon notifies the player when somebody enters or leaves the sector

-- by mr spuck - modified to notify via PM for dullbot/remotepilot by Nyscersul

spuck = spuck or {}

local MODE_NONE = 0
local MODE_PLAYER = 1
local MODE_ALL = 2

local mode_patterns = {}
mode_patterns[MODE_PLAYER] = "^[^*]"
mode_patterns[MODE_ALL] = ""


local SOUND_OFF = 0
local SOUND_ENTERED = 1
local SOUND_LEFT = 2
local SOUND_BOTH = 3


local enterleave = {}
enterleave.newchars = {}

-- get settings
local notify_mode = gkini.ReadInt("*enterleave", "notification_mode", MODE_PLAYER)
local notify_mode_pattern = mode_patterns[notify_mode]

local sound = gkini.ReadInt("*enterleave", "sound", SOUND_ENTERED)
local patterns = unspickle(gkini.ReadString("*enterleave", "patterns", ""))


function enterleave:EnterNotify(name)
	if sound == SOUND_ENTERED or sound == SOUND_BOTH then gksound.GKPlaySound("prox.warning") end
	--print("\127ffffff" .. name .." has entered the sector ") 
	local charid = GetCharacterIDByName(name)
	local player_name = GetPlayerName(charid)
	local player_faction = GetPlayerFaction(charid)
	if player_faction == 3 then
		player_faction= "UIT"
	elseif player_faction == 1 then
		player_faction= "Itani"
	elseif player_faction == 2 then
		player_faction= "Serco"
	else
		player_faction=""
	end
	local player_ship = GetPrimaryShipNameOfPlayer(charid) or ""
	local guild_tag = GetGuildTag(charid) or ""
	local itani_standing = GetPlayerFactionStanding(1, charid)
	local serco_standing = GetPlayerFactionStanding(2, charid)
	local uit_standing = GetPlayerFactionStanding(3, charid)
	local local_standing = GetPlayerFactionStanding("sector", charid)
	local tempcolour = ""
	if guild_tag~="" then
		guild_tag=(" [ "..guild_tag.." ], ")
	else
		guild_tag=""
	end
	if local_standing==0 or local_standing==nil then
		local_standing="none"
	end
	local sector_id = GetCurrentSectorid()
	local location=AbbrLocationStr(sector_id)
	location=(location.."("..sector_id..")")
	local friendly=GetFriendlyStatus(charid)
	--print("friendly status")
	--print(friendly)
	if friendly ~=3 then
		tempcolour="Hostile"
	else
		tempcolour="Friendly"
	end
	if player_ship == "" then
		player_ship=""
	else
		player_ship=("piloting : "..player_ship)
	end
	local tempstring= (" "..name .." has entered the sector, "..player_faction.." "..guild_tag..""..player_ship.." in "..location.." : "..tempcolour)--" Local Standing : "..local_standing)  --6..",  "..tempcolour)
	ProcessEvent("MESSAGE_WAITING",tempstring)
end

function enterleave:LeaveNotify(name)
	if sound == SOUND_LEFT or sound == SOUND_BOTH then gksound.GKPlaySound("prox.warning") end
	--print("\127ffffff" .. name .." has left the sector") 
	local tempstring= (" "..name .." has left the sector ")
	ProcessEvent("MESSAGE_WAITING",tempstring)
end

function enterleave:OnEvent(event, data)
	local function isname(name, charid)
		if name == "(reading transponder "..charid..")" then
			return false
		else
			return true
		end
	end

	local function matches_pattern(name)
		-- check if mode is valid and return true if it matches
		if notify_mode_pattern and name:match(notify_mode_pattern) then
			return true
		end
	end

	local function matches_custom_pattern(name)
		for _,pattern in ipairs(patterns) do
			if name:match(pattern) then return true end
		end
	end


	local playername = GetPlayerName(data) or "none"
		
	if event == "PLAYER_ENTERED_SECTOR" then
		-- don't do anything if the char is the player or sector
		if data == GetCharacterID() or data == 0 then
			return
		end

		-- custom patterns override other rules
		if matches_custom_pattern(playername) then
			self:EnterNotify(playername)
		-- if this is not a name put it into newplayer table till it's updated
		elseif not isname(playername, data) then
			self.newchars[data] = true
		elseif matches_pattern(playername) then
			self:EnterNotify(playername)
		end
	end

	if event == "PLAYER_LEFT_SECTOR" then
		-- do nothing if it's the sector (not sure if this actually happens)
		if data == 0 then return end

		-- pattern matching
		if matches_custom_pattern(playername) then
			self:LeaveNotify(playername)
		-- clear temporary char list when the player leaves sector. to avoid an ever growing table in case some get lost
		elseif data == GetCharacterID() then 
			self.newchars = {}
		-- notify if it's a name
		elseif isname(playername, data) and matches_pattern(playername) then
		       self:LeaveNotify(playername)
		end

		self.newchars[data] = nil
	end
	
	if event == "UPDATE_CHARINFO" then
		if self.newchars[data] then
			-- still don't have name so leave it in the table
			if not isname(playername, data) then
				return
			end
			
			self.newchars[data] = nil

			-- pattern matching
			if matches_custom_pattern(playername) or matches_pattern(playername) then
				self:EnterNotify(playername)
			end
		end
	end	

end


RegisterEvent(enterleave, "PLAYER_ENTERED_SECTOR")
RegisterEvent(enterleave, "PLAYER_LEFT_SECTOR")
RegisterEvent(enterleave, "UPDATE_CHARINFO")


local function notifymode_func(_, args)
	if args and args[1] then 
		local mode = tonumber(args[1]) or notify_mode
		local tempstring=""
		if mode == 0 then
			tempstring= ("Enter-Leave notifications off.")
		elseif mode == 1 then
			tempstring= ("Enter-Leave set to notify on Players only.")
		elseif mode == 2 then
			tempstring= ("Enter-Leave set to notify on Bots and Players.")
		end
		ProcessEvent("MESSAGE_WAITING",tempstring)
		gkini.WriteInt("*enterleave", "notification_mode", mode)
		notify_mode = mode
		notify_mode_pattern = mode_patterns[mode]
	else
		print("usage: /enterleave_notify_mode <number>\n 0 = no notification\n 1 = only players\n 2 = everything")
	end 
end


local function pattern_func(_, args)
	if args and args[1] then 
		gkini.WriteString("*enterleave", "patterns", spickle(args))
		patterns = args
	else
		gkini.WriteString("*enterleave", "patterns", "")
		patterns = {}
	end
end


local function sound_func(_, args)
	if args and args[1] then 
		local mode = tonumber(args[1]) or sound
		gkini.WriteInt("*enterleave", "sound", mode)
		sound = mode
	else
		print("usage: /enterleave_sound <number>\n 0 = off\n 1 = enter\n 3 = leave\n 4 = both")
	end 
end
RegisterUserCommand("enterleave_sound", sound_func)
RegisterUserCommand("el_reports", notifymode_func)
RegisterUserCommand("enterleave_pattern", pattern_func)
spuck.enterleave = enterleave
